/* parser.c - implements the file-parsing routines for dyn-host-filter and dhf-test
              Copyright (C) 2006, 2007 Simeon Simeonov


 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software Foundation,
 Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA. */


#include "parser.h"

#ifdef ENABLE_LOGGING
#include <time.h>
#endif

#define LINE_BUFFER_SIZE 1024 

static char *dhf_strip_regexp(char *str);

int
parse_dhf_rules(DHFConf *config, LinkedList *list)
{

  int i;
  char line[LINE_BUFFER_SIZE];

  char *line_ptr;

  FILE *rule_file;

#ifdef ENABLE_LOGGING
  FILE *log_file;
  char tbuffer[LINE_BUFFER_SIZE]; /* holds the timestamp string */
  
  time_t curtime;
  struct tm *loctime;

  curtime = time (NULL);
  loctime = localtime (&curtime);
  strftime (tbuffer, LINE_BUFFER_SIZE, "%d:%m:%Y %H:%M:%S", loctime);

  if ((log_file = fopen(DHF_LOG_FILE, "a")) == NULL) {
    return -1;
  }
#endif


  if ((rule_file = fopen(DHF_CONF_FILE, "r")) == NULL) {
#ifdef ENABLE_LOGGING
    fprintf(log_file, "%s %s %s %s %s. %s\n", tbuffer, DHFSTR_ERROR, DHFSTR_ERROR_OPEN, DHF_CONF_FILE, DHFSTR_ERROR_READ, DHFSTR_ERROR_TERMINATE);
#endif
    return -1;
  }

  line_ptr = line;

  for (i = 1; ((fgets((char *) line, LINE_BUFFER_SIZE, rule_file)) != NULL); i++) {


    if ((strncmp(line_ptr, "pass", 4)) == 0) {

      regex_t compiled;
      if (regcomp (&compiled, dhf_strip_regexp(line_ptr + 5), REG_ICASE | REG_EXTENDED) != 0) {
#ifdef ENABLE_LOGGING
        fprintf(log_file, "%s %s %s %s:%d. %s\n", tbuffer, DHFSTR_ERROR, DHFSTR_ERROR_INVALID_REGEXP, DHF_CONF_FILE, i, DHFSTR_ERROR_TERMINATE);
#endif
        return -1;
      }

      list_insert(list, compiled, DHF_ACTION_PASS);

    }


    else if ((strncmp(line_ptr, "block", 5)) == 0) {

      regex_t compiled;
      if (regcomp (&compiled, dhf_strip_regexp(line_ptr + 6), REG_ICASE | REG_EXTENDED) != 0) {
#ifdef ENABLE_LOGGING
        fprintf(log_file, "%s %s %s %s:%d. %s\n", tbuffer, DHFSTR_ERROR, DHFSTR_ERROR_INVALID_REGEXP, DHF_CONF_FILE, i, DHFSTR_ERROR_TERMINATE);
#endif
        return -1;
      }

      list_insert(list, compiled, DHF_ACTION_BLOCK);

    }

    else if ((strncmp(line_ptr, "default_action", 14)) == 0) {

      if ((strncmp(line_ptr + 15, "block", 5)) == 0)
        config->default_action = DHF_ACTION_BLOCK;
      /* otherwise use the default setting (already assigned to config) */
    }

    else if ((strncmp(line_ptr, "default_unresolved_action", 25)) == 0) {
      
      if ((strncmp(line_ptr + 26, "pass", 4)) == 0)
        config->default_unresolved_action = DHF_ACTION_PASS;
      /* otherwise use the default setting (already assigned to config) */
    } 

    else if ((strncmp(line_ptr, "default_ip_action", 17)) == 0) {

      if ((strncmp(line_ptr + 18, "pass", 4)) == 0)
        config->default_ip_action = DHF_ACTION_PASS;
      /* otherwise use the default setting (already assigned to config) */
    }

    else if ((strncmp(line_ptr, "host_code", 9)) == 0) {

      if ((strncmp(line_ptr + 10, "451", 3)) == 0)
        config->block_host_code = DHF_SMTP_451;
      /* otherwise use the default setting (already assigned to config) */
    }

    else if ((strncmp(line_ptr, "unresolved_code", 15)) == 0) {

      if ((strncmp(line_ptr + 16, "553", 3)) == 0)
        config->block_unresolved_code = DHF_SMTP_553;
      /* otherwise use the default setting (already assigned to config) */
    }

    else if ((strncmp(line_ptr, "ip_code", 7)) == 0) {

      if ((strncmp(line_ptr + 8, "553", 3)) == 0)
        config->block_ip_code = DHF_SMTP_553;
      /* otherwise use the default setting (already assigned to config) */
    }

  }

  fclose(rule_file);

#ifdef ENABLE_LOGGING
  fclose(log_file);
#endif

  return 0;

}


int 
parse_dhf_test_rules(const char *config_file, DHFConf *config, LinkedList *list)
{

  int i;
  char line[LINE_BUFFER_SIZE];

  char *line_ptr;

  FILE *rule_file;

  if ((rule_file = fopen(config_file, "r")) == NULL) {
    fprintf (stderr, "%s %s %s %s. %s\n", DHFSTR_ERROR, DHFSTR_ERROR_OPEN, config_file, DHFSTR_ERROR_READ, DHFSTR_ERROR_TERMINATE);
    return -1;
  }

  line_ptr = line;

  for (i = 1; ((fgets((char *) line, LINE_BUFFER_SIZE, rule_file)) != NULL); i++) {

    if ((strncmp(line_ptr, "pass", 4)) == 0) {

      regex_t compiled;
      if (regcomp (&compiled, dhf_strip_regexp(line_ptr + 5), REG_ICASE | REG_EXTENDED) != 0) {
        fprintf (stderr, "%s %s %s:%d. %s\n", DHFSTR_ERROR, DHFSTR_ERROR_INVALID_REGEXP, config_file, i, DHFSTR_ERROR_TERMINATE);
        return -1;
      }

      list_insert_test(list, line_ptr + 5,compiled, DHF_ACTION_PASS);

    }


    else if ((strncmp(line_ptr, "block", 5)) == 0) {

      regex_t compiled;
      if (regcomp (&compiled, dhf_strip_regexp(line_ptr + 6), REG_ICASE | REG_EXTENDED) != 0) {
        fprintf (stderr, "%s %s %s:%d. %s\n", DHFSTR_ERROR, DHFSTR_ERROR_INVALID_REGEXP, config_file, i, DHFSTR_ERROR_TERMINATE);
        return -1;
      }

      list_insert_test(list, line_ptr + 6, compiled, DHF_ACTION_BLOCK);

    }

    else if ((strncmp(line_ptr, "default_action", 14)) == 0) {

      if ((strncmp(line_ptr + 15, "block", 5)) == 0)
        config->default_action = DHF_ACTION_BLOCK;
      /* otherwise use the default setting (already assigned to config) */
    }

    else if ((strncmp(line_ptr, "default_unresolved_action", 25)) == 0) {
      
      if ((strncmp(line_ptr + 26, "pass", 4)) == 0)
        config->default_unresolved_action = DHF_ACTION_PASS;
      /* otherwise use the default setting (already assigned to config) */
    } 

    else if ((strncmp(line_ptr, "default_ip_action", 17)) == 0) {

      if ((strncmp(line_ptr + 18, "pass", 4)) == 0)
        config->default_ip_action = DHF_ACTION_PASS;
      /* otherwise use the default setting (already assigned to config) */
    }

    else if ((strncmp(line_ptr, "host_code", 9)) == 0) {

      if ((strncmp(line_ptr + 10, "451", 3)) == 0)
        config->block_host_code = DHF_SMTP_451;
      /* otherwise use the default setting (already assigned to config) */
    }

    else if ((strncmp(line_ptr, "unresolved_code", 15)) == 0) {

      if ((strncmp(line_ptr + 16, "553", 3)) == 0)
        config->block_unresolved_code = DHF_SMTP_553;
      /* otherwise use the default setting (already assigned to config) */
    }

    else if ((strncmp(line_ptr, "ip_code", 7)) == 0) {

      if ((strncmp(line_ptr + 8, "553", 3)) == 0)
        config->block_ip_code = DHF_SMTP_553;
      /* otherwise use the default setting (already assigned to config) */
    }

  }

  fclose(rule_file);

  return 0;

}


static char * 
dhf_strip_regexp(char *str)
{

  int i;

  for (i = 0; i < strlen(str); i++) {

    if (str[i] == '\n') {
      str[i] = '\0';
    } 

  }

  return str;

}
